@@ -18,6 +18,8 @@ pub mod track;
 pub mod util;
 
 pub mod prelude {
+    #[doc(hidden)]
+    pub use notation_proto::prelude::*;
     #[doc(hidden)]
     pub use crate::bar::{Bar, BarLayer};
     #[doc(hidden)]
@@ -39,6 +41,8 @@ pub mod prelude {
     #[doc(hidden)]
     pub use crate::play::play_state::{PlayState, PlayingState};
     #[doc(hidden)]
+    pub use crate::play::play_events::*;
+    #[doc(hidden)]
     pub use crate::section::Section;
     #[doc(hidden)]
     pub use crate::tab::Tab;
@@ -49,7 +53,15 @@ pub mod prelude {
     #[doc(hidden)]
     pub use crate::track::{Track, TrackProps};
     #[doc(hidden)]
+    pub use notation_proto::prelude::Bar as ProtoBar;
+    #[doc(hidden)]
+    pub use notation_proto::prelude::BarLayer as ProtoBarLayer;
+    #[doc(hidden)]
     pub use notation_proto::prelude::Tab as ProtoTab;
     #[doc(hidden)]
-    pub use notation_proto::prelude::*;
+    pub use notation_proto::prelude::Track as ProtoTrack;
+    #[doc(hidden)]
+    pub use notation_proto::prelude::Section as ProtoSection;
+    #[doc(hidden)]
+    pub use notation_proto::prelude::Form as ProtoForm;
 }
